#!/bin/bash
#

# buildLhapdf.sh <product_directory> <e7|e6> <debug|opt|prof> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e7|e6> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    patchdir=${product_dir}/${package}/${pkgver}/patch
    scriptdir=${product_dir}/${package}/${pkgver}/scripts
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=lhapdf
origpkgver=v5_9_1
pkgver=${origpkgver}c
ssibuildshims_version=v0_08_06
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cxxflg="-O3 -g -DNDEBUG"
elif [ "${extraqual}" = "debug" ]
then
    cxxflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cxxflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

if [[ "${basequal}" == "e[45]" ]]
then
  cxxflg="${cxxflg} -std=c++11 -Wno-deprecated-declarations"
elif [[ "${basequal}" == e[67] ]]
then
  cxxflg="${cxxflg} -std=c++1y -Wno-deprecated-declarations"
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# make sure pdfsets is installed
${scriptdir}/getPdfSets.sh ${product_dir} || die "ERROR: pdfsets not available"

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} 

if [ -z ${LHAPATH} ]
then
   echo "ERROR: failed to setup pdfsets"
   exit 1
else
   echo "LHAPATH: ${LHAPATH}"
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

if [ ! -d ${sourcedir}/${pkgtarname} ]
then
   cd ${sourcedir}
   tar xf ${tardir}/${pkgtarname}.tar.gz
fi

cd ${blddir}

CXXFLAGS="${cxxflg}" ${sourcedir}/${pkgtarname}/configure \
 --prefix=${pkgdir} \
 --enable-low-memory --disable-pyext --disable-octave || \
 die "ERROR: configure failed."
make -j ${ncore} || die "ERROR: make failed."
make check || die "ERROR: make check failed"
make install || die "ERROR: make install failed."

# install example source code by hand
cp -pr ${sourcedir}/${pkgtarname}/examples ${pkgdir}/examples
rm -f ${pkgdir}/examples/Makefile.*
rm -f ${pkgdir}/examples/*.py
cp -p ${blddir}/examples/* ${pkgdir}/examples/

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || exit 1

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "lhapdf is installed at ${LHAPDF_FQ_DIR}"

# fixing bin/lhapdf-config
mv ${pkgdir}/bin/lhapdf-config ${pkgdir}/bin/lhapdf-config.bak
cat ${pkgdir}/bin/lhapdf-config.bak | sed "s%${LHAPDF_FQ_DIR}%\${LHAPDF_FQ_DIR}%g" > ${pkgdir}/bin/lhapdf-config

# remove libtool files since they contain hardwired paths
rm -vf ${LHAPDF_FQ_DIR}/lib/*.la

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
